@@ -25,6 +25,10 @@ class PinotExit(Exception):
     pass
 
 
+class PinotAlert(Exception):
+    pass
+
+
 import urwid.curses_display
 import urwid
 import threading
@@ -117,7 +121,10 @@ class RetroMainLoop(object):
                     if self._unhandled_input is not None:
                         key = self._unhandled_input(key)
                     if key is not None:
-                        self.view.keypress(size, key)
+                        try:
+                            self.view.keypress(size, key)
+                        except PinotAlert as inst:
+                            self.display_notification(str(inst))
 
     def on_init(self):
         pass
