@@ -16,15 +16,6 @@ class JsonSchemaError(RuntimeError):
 
 
 
-class AdditionalPropertiesKey(object):
-    """
-    Dummy class used as a key for additionalProperties nodes.  This library 
-    uses an object instance rather than a string as a key to prevent collisions
-    with valid JSON keys.
-    """
-    pass
-
-
 class SchemaNode(JsonBaseNode):
     """
     Each SchemaNode instance represents one node in the data tree.  Each
@@ -33,7 +24,7 @@ class SchemaNode(JsonBaseNode):
     """
 
     def __init__(self, key=None, data=None, filename=None, parent=None, 
-                 ordermap=None, fmt=None):
+                 ordermap=None, fmt=None, isaddedprop=False):
 
         if filename is not None:
             self.filename = filename
@@ -57,6 +48,8 @@ class SchemaNode(JsonBaseNode):
         properties_id = fmt.idmap['properties']
         items_id = fmt.idmap['items']
 
+        self.is_added_prop = isaddedprop
+
         # object ref for the parent
         self.parent = parent
         # local index for the node
@@ -67,15 +60,27 @@ class SchemaNode(JsonBaseNode):
         else:
             self.depth = self.parent.get_depth() + 1
             self.rootschema = self.parent.get_root_schema()
+
+        self._register_fragment_id()
+
         if(self.data['type'] == fmt.typemap['object']):
             self.children = {}
             
             for subkey, subdata in self.data[properties_id].items():
                 ordermap = \
                     self.ordermap['children'][properties_id]['children'][subkey]
+
+                if (self.schemaformat.version == 1 and 
+                    'user_key' in self.data and
+                    subkey == self.data['user_key']):
+                    isaddedprop = True
+                else:
+                    isaddedprop = False
+
                 self.children[subkey] = SchemaNode(key=subkey, data=subdata,
                                                    parent=self, 
-                                                   ordermap=ordermap)
+                                                   ordermap=ordermap,
+                                                   isaddedprop=isaddedprop)
         elif(self.data['type'] == fmt.typemap['array']):
             if fmt.version == 1:
                 ordermap = self.ordermap['children'][items_id]['children'][0]
@@ -86,6 +91,35 @@ class SchemaNode(JsonBaseNode):
                 self.children = [SchemaNode(key=0, data=self.data[items_id][0],
                                             parent=self, ordermap=ordermap)]
 
+    def _register_fragment_id(self):
+        """
+        If this schema node has a globally addressable id, then register 
+        it with the root schema.  This probably only makes sense with v1 
+        schemas.
+        """
+        if self.schemaformat.version == 1 and 'id' in self.data:
+            root = self.get_root_schema()
+            root._store_node_as_id(self, self.data['id'])
+
+    def _store_node_as_id(self, node, id):
+        """
+        Add node to global id index
+        """
+        try:
+            self._node_index[id]=node
+        except AttributeError:
+            self._node_index={id:node}
+
+    def _get_node_by_id(self, id):
+        try:
+            return self._node_index[id]
+        except KeyError, AttributeError:
+            return None
+    
+    def resolve_fragment_id(self):
+        """ if this is a fragment ref/idref, return the target schema node """
+        return self.get_root_schema()._get_node_by_id(self.data['idref'])
+
     def get_depth(self):
         return self.depth
 
@@ -108,6 +142,12 @@ class SchemaNode(JsonBaseNode):
     def set_title(self, title):
         self.data['title'] = title
 
+    def get_description(self):
+        if 'description' in self.data:
+            return self.data['description']
+        else:
+            return ''
+
     def get_type(self):
         return self.data['type']
     
@@ -146,7 +186,13 @@ class SchemaNode(JsonBaseNode):
 
     def get_child(self, key):
         if(self.is_type('object')):
-            return self.children[key]
+            try:
+                return self.children[key]
+            except KeyError:
+                if(self.allow_additional_properties()):
+                    return self.get_additional_props_node()
+                else:
+                    raise
         elif(self.is_type('array')):
             return self.children[0]
         else:
@@ -184,7 +230,14 @@ class SchemaNode(JsonBaseNode):
         if self.allow_additional_properties():
             if self.schemaformat.version == 1:
                 userkey = self.data['user_key']
-                return self.data['mapping'][userkey]
+                propdata = self.data['mapping'][userkey]
+                ordermap = \
+                    self.ordermap['children']['mapping']['children'][userkey]
+                self.additional_props = SchemaNode(data=propdata, parent=self, 
+                                                   ordermap=ordermap, 
+                                                   key=userkey,
+                                                   isaddedprop=True)
+                return self.additional_props
             elif self.schemaformat.version == 2:
                 return self.get_additional_props_node_v2()
             else:
@@ -200,13 +253,13 @@ class SchemaNode(JsonBaseNode):
             else:
                 propdata = {}
                 ordermap = {'keys':[], 'children':{}}
-            key = AdditionalPropertiesKey()
             self.additional_props = SchemaNode(data=propdata, parent=self, 
-                                               ordermap=ordermap, key=key)
+                                               ordermap=ordermap,
+                                               isaddedprop=True)
         return self.additional_props
         
     def is_additional_props_node(self):
-        return isinstance(self.key, AdditionalPropertiesKey)
+        return self.is_added_prop
 
     def enum_options(self):
         return self.data['enum']
