@@ -227,15 +227,21 @@ class ParentWidget(TreeWidget):
         if not self.expanded: 
             return None
         else:
-            firstnode = self._node.get_first_child()
-            return firstnode.get_widget()
+            if self._node.has_children():
+                firstnode = self._node.get_first_child()
+                return firstnode.get_widget()
+            else:
+                return None
 
     def last_child(self):
         """Return last child if expanded."""
         if not self.expanded:
             return None
         else:
-            lastchild = self._node.get_last_child().get_widget()
+            if self._node.has_children():
+                lastchild = self._node.get_last_child().get_widget()
+            else:
+                return None
             # recursively search down for the last descendant
             lastdescendant = lastchild.last_child()
             if lastdescendant is None:
@@ -287,6 +293,12 @@ class TreeNode(object):
     def get_key(self):
         return self._key
 
+    def set_key(self, key):
+        self._key = key
+
+    def change_key(self, key):
+        self.get_parent().change_child_key(self._key, key)
+
     def get_parent(self):
         if self._parent == None and self.get_depth() > 0:
             self._parent = self.load_parent()
@@ -365,6 +377,12 @@ class ParentNode(TreeNode):
         """Set the child node for a given key.  Useful for bottom-up, lazy 
         population of a tree.."""
         self._children[key]=node
+    
+    def change_child_key(self, oldkey, newkey):
+        if newkey in self._children:
+            raise TreeWidgetError("Key '%s' is already in use" % newkey)
+        self._children[newkey] = self._children.pop(oldkey)
+        self._children[newkey].set_key(newkey)
 
     def get_child_index(self, key):
         try:
@@ -415,7 +433,10 @@ class ParentNode(TreeNode):
         """Return the last TreeNode in the directory."""
         child_keys = self.get_child_keys()
         return self.get_child_node(child_keys[-1])
-
+        
+    def has_children(self):
+        """Does this node have any children?"""
+        return len(self.get_child_keys())>0
 
 class TreeWalker(urwid.ListWalker):
     """ListWalker-compatible class for browsing directories.
